<?php

namespace UnicaenDbAnonym\Service;

use Generator;
use UnicaenApp\Service\SQL\RunSQLResult;
use Webmozart\Assert\Assert;
use Webmozart\Assert\InvalidArgumentException;

/**
 * @property \Psr\Log\LoggerInterface $logger
 */
class AnonymService
{
    use DbServiceAwareTrait;

    /**
     * @var array
     */
    protected array $entitiesConfig = [];

    /**
     * @var array
     */
    protected array $outputConfig = [];

    /**
     * @param array $config
     * @return self
     */
    public function setConfig(array $config): self
    {
        $this->entitiesConfig = $config['entities'];
        $this->outputConfig = $config['output'];

        return $this;
    }

    public function getAnonymisationScriptPath()
    {
        return $this->outputConfig['anonymisation'];
    }

    public function getRestaurationScriptPath()
    {
        return $this->outputConfig['restauration'];
    }

    public function generer(): Generator
    {
        if ($this->dbService->isBddAnonymisee()) {
            throw new InvalidArgumentException(
                "Génération interdite car la base de données est déjà anonymisée (cf. témoin dans la table des metadata)");
        }

        $anonymisationFilePath = $this->getAnonymisationScriptPath();
        $restaurationFilePath = $this->getRestaurationScriptPath();

        if (file_exists($anonymisationFilePath)) {
            throw new InvalidArgumentException("Le fichier '$anonymisationFilePath' existe déjà !");
        }
        if (file_exists($restaurationFilePath)) {
            throw new InvalidArgumentException("Le fichier '$restaurationFilePath' existe déjà !");
        }

        Assert::notEmpty($this->entitiesConfig, "Aucune classe d'entité trouvée dans la config");

        $mf = fopen($anonymisationFilePath, 'w');
        $rf = fopen($restaurationFilePath, 'w');

        foreach ($this->entitiesConfig as $entityName => $entityConfig) {

            Assert::keyExists($entityConfig, $k = 'mapping', "La config de l'entité '$entityName' doit posséder la clé '$k'");

            $metadata = $this->dbService->getEntityClassMetadata($entityName);
            $mapping = $entityConfig['mapping'];
            $except = $entityConfig['except'] ?? [];
            $fields = array_keys($mapping);

            $records = $this->dbService->fetchEntityRecords($entityName, $fields, $except);

            $migrateSqlLines = [];
            $restaurationSqlLines = [];

            foreach ($records as $record) {
                $migrateSqlLines[] = $this->dbService->genUpdateForAnonymiser($record, $mapping, $metadata);
                $restaurationSqlLines[] = $this->dbService->genUpdateForRestaurer($record, $mapping, $metadata);
            }

            fputs($mf, implode(PHP_EOL, $migrateSqlLines) . PHP_EOL);
            fputs($rf, implode(PHP_EOL, $restaurationSqlLines) . PHP_EOL);

            yield [
                'table' => $metadata->getTableName(),
                'fields' => $fields,
                'count' => count($records),
            ];

            unset ($records);
        }

        fclose($mf);
        fclose($rf);
    }

    /**
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Doctrine\DBAL\Exception
     */
    public function anonymiser(): RunSQLResult
    {
        if ($this->dbService->isBddAnonymisee()) {
            throw new InvalidArgumentException(
                "Anonymisation interdite car la base de données est déjà anonymisée (cf. témoin dans la table des metadata)");
        }

        $scriptPath = $this->getAnonymisationScriptPath();
        $result = $this->dbService->lancerScript($scriptPath);
        $this->dbService->setBddAnonymisee(true);

        return $result;
    }

    /**
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Doctrine\DBAL\Exception
     */
    public function restaurer(): RunSQLResult
    {
        if (! $this->dbService->isBddAnonymisee()) {
            throw new InvalidArgumentException(
                "Restauration interdite car la base de données n'est pas anonymisée (cf. témoin dans la table des metadata)");
        }

        $scriptPath = $this->getRestaurationScriptPath();
        $result = $this->dbService->lancerScript($scriptPath);
        $this->dbService->setBddAnonymisee(false);

        return $result;
    }
}
